'''
Created on Jan 15, 2015

@author: marten
'''
from sqlalchemy import func
from sqlalchemy.orm.util import aliased
from django.utils.translation import ugettext_lazy as _
from basesoft.web.contrib.i18n.models import Country
from basesoft.web.contrib.license.models import License

from we.core.custom_query import AbstractWeCustomQuery
from we.member.models import WeHqppLicense, WeLpp

from basesoft.web.core.util.custom_query import CustomQueryModel, CustomQueryField
from basesoft.web.contrib.contact.models import Company, Address, CompanyAddress, BankAccount, CompanyBankAccount, Contact, Person

LppCompany = aliased(Company)
ArCompany = aliased(Company)
InvoiceCompany = aliased(Company)
TbsCompany = aliased(Company)
TbsPerson = aliased(Person)
Tbs = aliased(License) # we need to alias this, cuz both TbsLicense and WeHqppLicense inherit License.

class MemberCustomQuery(AbstractWeCustomQuery):

    permissions = ('we_view_member_license', )

    def tables(self):
        return (
            CustomQueryModel(WeHqppLicense),
            CustomQueryModel(Company, onclause=WeHqppLicense.company_id ==Company.id),
            CustomQueryModel(Person, onclause=Person.id == Company.contact_person_id, use_outerjoin=True),
            CustomQueryModel(WeLpp, onclause=WeHqppLicense.license_id == WeLpp.hqpp_id, use_outerjoin=True),
            CustomQueryModel(LppCompany, onclause=WeLpp.company_id == LppCompany.id, use_outerjoin=True),
            CustomQueryModel(ArCompany, onclause=WeLpp.ar_company_id == ArCompany.id, use_outerjoin=True),
            CustomQueryModel(InvoiceCompany, onclause=WeLpp.invoice_company_id == InvoiceCompany.id, use_outerjoin=True),
            CustomQueryModel(Tbs, onclause=WeLpp.tbs_id == Tbs.id, use_outerjoin=True),
            CustomQueryModel(TbsCompany, onclause=TbsCompany.id == Tbs.company_id, use_outerjoin=True),
            CustomQueryModel(TbsPerson, onclause=TbsPerson.id == TbsCompany.contact_person_id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(WeHqppLicense.id, _('ID'), 'member_id'),
            CustomQueryField(WeHqppLicense.name, _('Producer'), 'member_name', is_filter=True),
            CustomQueryField(Company.vat_no, _('Producer VAT'), 'member_vat', is_filter=True, use_when_separated=False),
            CustomQueryField(func.concat_ws(' ', Person.first_name, Person.middle_name, Person.last_name),
                             _('Producer contact'), key='hqpp_contact', use_when_separated=False),
            CustomQueryField(Person.email, _('Producer contact e-mail'), 'hqpp_contact_email', is_filter=True, use_when_separated=False),
            CustomQueryField(Person.phone, _('Producer contact phone'), 'hqpp_contact_phone', is_filter=True, use_when_separated=False),
            CustomQueryField(False, _('Local producer partner'), 'lpp', is_filter=True, use_when_separated=WeLpp.name),
            CustomQueryField(False, _('LPP ID'), 'lpp_id', use_when_separated=WeLpp.id),
            CustomQueryField(False, _('LPP company id'), 'lpp_company_id', use_when_separated=WeLpp.company_id),
            CustomQueryField(False, _('LPP VAT no.'), 'lpp_vat_no', is_filter=True, use_when_separated=LppCompany.vat_no),
            CustomQueryField(False, _('TBS'), 'tbs', is_filter=True, use_when_separated=Tbs.name),
            CustomQueryField(False, _('TBS contact'), key='tbs_contact',
                             use_when_separated=func.concat_ws(' ', TbsPerson.first_name, TbsPerson.middle_name, TbsPerson.last_name)),
            CustomQueryField(False, _('TBS contact e-mail'), 'tbs_contact_email', is_filter=True, use_when_separated=TbsPerson.email),
            CustomQueryField(False, _('TBS contact phone'), 'tbs_contact_phone', is_filter=True, use_when_separated=TbsPerson.phone),
            CustomQueryField(False, _('LPP status'), 'status', is_filter=True, filter_options=[{
                'value': WeLpp.STATUS_NEW,
                'label': _("New"),
            },{
                'value': WeLpp.STATUS_PENDING,
                'label': _("Pending"),
            },{
                'value': WeLpp.STATUS_ACTIVE,
                'label': _("Active"),
            },{
                'value': WeLpp.STATUS_INACTIVE,
                'label': _("Inactive"),
            }], use_when_separated=WeLpp.status),
            CustomQueryField(False, _('LPP relation id'), 'lpp_relation_id',
                             use_when_separated=WeLpp.lpp_relation_id),
            CustomQueryField(False, _('Invoice company relation id'), 'invoice_relation_id',
                             use_when_separated=WeLpp.invoice_relation_id),
            CustomQueryField(False, _('Authorized representative'), 'ar_company', is_filter=True,
                             use_when_separated=ArCompany.name),
            CustomQueryField(False, _('Authorized representative id'), 'ar_company_id', is_filter=True,
                             is_hidden=True, use_when_separated=ArCompany.id),
            CustomQueryField(False, _('Invoice company'), 'invoice_company', is_filter=True,
                             use_when_separated=InvoiceCompany.name),
            CustomQueryField(False, _('Invoice company id'), 'invoice_company_id', is_filter=True,
                             is_hidden=True, use_when_separated=InvoiceCompany.id),

        )

    def set_where(self):
        # operators can see everyone, no need for slow IN-statements.
        if self.has_permissions('we_create_member_license'):
            return
        members = self.get_member_licenses()
        self.where = WeHqppLicense.id.in_([m.id for m in members])

    def set_group_by(self):
        self.group_by = None
        if self.separated is False:
            self.group_by = (WeHqppLicense.id, )

BankAddress = aliased(Address)
LppAddress = aliased(Address)
BankCountry = aliased(Country)


class LppMasterDataCustomQuery(AbstractWeCustomQuery):

    permissions = ('we_view_lpp_companies', )

    def tables(self):
        return (
            CustomQueryModel(WeHqppLicense),
            CustomQueryModel(WeLpp, onclause=WeHqppLicense.license_id == WeLpp.hqpp_id),
            CustomQueryModel(Company, onclause=WeLpp.company_id == Company.id, use_outerjoin=True),
            CustomQueryModel(CompanyAddress, onclause=CompanyAddress.company_id == Company.id, use_outerjoin=True),
            CustomQueryModel(LppAddress, onclause=LppAddress.id == CompanyAddress.address_id, use_outerjoin=True),
            CustomQueryModel(Country, onclause=Country.id == LppAddress.country_id, use_outerjoin=True),
            CustomQueryModel(CompanyBankAccount, onclause=CompanyBankAccount.company_id == Company.id, use_outerjoin=True),
            CustomQueryModel(BankAccount, onclause=CompanyBankAccount.bank_account_id == BankAccount.id, use_outerjoin=True),
            CustomQueryModel(BankAddress, onclause=BankAddress.id == BankAccount.address_id, use_outerjoin=True),
            CustomQueryModel(BankCountry, onclause=BankCountry.id == BankAddress.country_id, use_outerjoin=True),
            CustomQueryModel(Contact, onclause=Company.id == Contact.company_id, use_outerjoin=True),
            CustomQueryModel(Person, onclause=Person.id == Contact.person_id, use_outerjoin=True),
        )

    def fields(self):
        return (
            CustomQueryField(WeHqppLicense.name, _('Producer'), 'member_name', is_filter=True),
            CustomQueryField(WeLpp.name, _('Local producer partner'), 'lpp', is_filter=True),
            CustomQueryField(WeLpp.lpp_relation_id, _('LPP relation id'), 'lpp_relation_id', is_filter=True),
            CustomQueryField(Company.vat_no, _('VAT'), 'lpp_vat_no', is_filter=True),
            CustomQueryField(Company.website, _('Website'), 'lpp_website', is_filter=True),
            CustomQueryField(Country.name, _('Country'), 'country', is_filter=True),
            CustomQueryField(LppAddress.city, _('City'), 'city', is_filter=True),
            CustomQueryField(LppAddress.street, _('Street'), 'street', is_filter=True),
            CustomQueryField(LppAddress.postcode, _('Postal code'), 'postcode', is_filter=True),
            CustomQueryField(BankAccount.number, _('Bank account number'), 'bank_account_number', is_filter=True),
            CustomQueryField(BankAccount.iban, _('IBAN'), 'bank_iban', is_filter=True),
            CustomQueryField(BankAccount.bic, _('BIC'), 'bank_bic', is_filter=True),
            CustomQueryField(BankCountry.name, _('Bank country'), 'bank_country', is_filter=True),
            CustomQueryField(BankAddress.city, _('Bank city'), 'bank_city', is_filter=True),
            CustomQueryField(BankAddress.street, _('Bank street'), 'bank_street', is_filter=True),
            CustomQueryField(BankAddress.postcode, _('Bank postal code'), 'bank_postcode', is_filter=True),
            CustomQueryField(Person.first_name, _('Contact first name'), 'person_first_name', is_filter=True),
            CustomQueryField(Person.last_name, _('Contact last name'), 'person_last_name', is_filter=True),
            CustomQueryField(Person.email, _('Contact e-mail'), 'person_email', is_filter=True),
            CustomQueryField(Person.phone, _('Contact phone'), 'person_phone', is_filter=True),
        )

    def set_where(self):
        # operators can see everyone, no need for slow IN-statements.
        if self.has_permissions('we_create_member_license'):
            return
        lpps = self.get_local_producer_partners()
        self.where = WeLpp.id.in_([l.id for l in lpps])

    group_by = (WeLpp.company_id, )


custom_queries = (MemberCustomQuery, LppMasterDataCustomQuery, )
