'''
Created on Oct 14, 2014

@author: rado
'''
from basesoft.web.core.forms.handler import FormValidator
from basesoft.web.core.input.validation import plugins

from we.member.controllers import WeMemberController

class WeMemberRpcHandler:
    def __init__(self, request, **kwargs):
        self.sa_session = request.Session()
        self.account_id = request.session['user_id']
        self.controller = WeMemberController(self.sa_session, account_id=self.account_id, request=request)

    def load_member(self, member_id):
        return self.controller.load_member(member_id)

    def get_members(self):
        return self.controller.get_members()

    def save_member(self, _id, state):
        member = self.controller.save_or_update_member(_id, state)
        return member.id

    def register_member(self, master_data, account_data):
        member = self.controller.register_member(master_data, account_data)
        return {'member_name': member.name, 'member_id': member.id, 'company_id': member.company.id}

    def load_company_details(self, company_id):
        return self.controller.load_company_details(company_id)

    def load_member_lpps(self):
        lpps = self.controller.load_lpps()
        return sorted(lpps, key=lambda k: k['label'])

    def save_or_update_company_details(self, company_id, state):
        # company_id not used!
        # need to stick with the basesoft/contact/controller structure
        for key in state['company']:
            state[key] = state['company'][key]
        del state['company']
        self.controller.save_or_update_company_details(company_id, state)

    def remove_member_lpp(self, lpp_id):
        self.controller.delete_lpp(lpp_id)
        return True

    def get_tbs_and_currency(self, lpp_id):
        return self.controller.get_tbs_and_currency(lpp_id)

    def save_companies_linked_to_tbs(self, hqpp_id, state):
        return self.controller.save_companies_linked_to_tbs(hqpp_id, state)

    def get_companies_per_hqpp_linked_to_tbs(self, hqpp_id):
        r = self.controller.get_companies_per_hqpp_linked_to_tbs(hqpp_id)
        return {'tbs_links': sorted(r['tbs_links'], key=lambda k: k['tbs_name'])}

    def get_all_companies(self, filters, limit=20, hqpp_id=None, tbs_id=None):
        if limit > 100:
            limit = 100
        l = [{'label': c.name, 'value': c.id} for c in self.controller.get_all_companies(filters.get('filter'), filters.get('value', None), limit, hqpp_id, tbs_id)]
        return sorted(l, key=lambda k: k['label'])

    def get_ar_companies(self, filters, limit=20, hqpp_id=None, tbs_id=None):
        if limit > 100:
            limit = 100
        l = [{'label': c.name, 'value': c.id} for c in self.controller.get_ar_companies(filters.get('filter'), filters.get('value', None), limit, hqpp_id, tbs_id)]
        return sorted(l, key=lambda k: k['label'])

    def get_companies_acting_as_lpp(self, filters, limit=20, hqpp_id=None, exclude=None):
        if limit > 100:
            limit = 100
        l = [{'label': c.name, 'value': c.id} for c in
             self.controller.get_companies_acting_as_lpp(filters.get('filter'),
                                                         filters.get('value', None),
                                                         limit, hqpp_id, exclude)]
        return sorted(l, key=lambda k: k['label'])

    def get_all_tbs(self, hqpp_id):
        return self.controller.get_all_tbs(hqpp_id)

    def get_tbses_for_product_import(self, hqpp_id):
        return self.controller.get_tbses_for_product_import(hqpp_id)

    def save_lpp_status(self, lpp_id, status):
        self.controller.save_lpp_status(lpp_id, status)

    def validate_tbs(self, args):
        return FormValidator({
            'tbs_id': [(plugins.Required, {})],
        }).validate(args)

    def validate_tbs_links_form(self, args):
        form_handler = FormValidator({
            'tbs_id': [(plugins.Required, {})],
            'lpp_company_id': [(plugins.Required, {})],
        })

        return form_handler.validate(args)


handlers = (WeMemberRpcHandler,)
