'''
Created on Oct 10, 2014

@author: rado
'''
from sqlalchemy.orm.util import aliased
from sqlalchemy.sql.expression import case, and_, or_
from sqlalchemy import func

from django.utils.translation import ugettext as _

from basesoft.web.contrib.file.custom_query import AbstractFileImportCustomQuery
from basesoft.web.contrib.file.models import ImportFileType
from basesoft.web.core.exceptions import PublicSecurityException
from basesoft.web.contrib.contact.models import Company, CompanyAddress, Address
from basesoft.web.contrib.i18n.models import Country
from basesoft.web.contrib.license.models import License
from basesoft.web.core.util.custom_query import CustomQueryModel, CustomQueryField, DATA_TYPE_STRING, DATA_TYPE_DECIMAL

from iqportal.weee.core.custom_query import WEEEProductLevelCustomQuery
from iqportal.weee.core.models import WEEEProductLevel

from we.core.custom_query import AbstractWeCustomQuery
from we.declaration.importers import WeDeclarationImporter
from we.member.models import WeHqppLicense, WeLpp
from we.product.importers import HqppProductLineImporter, CN8LineImporter, TbsPriceSheetLineImporter, \
    CN8ToTbsProductLevelLineImporter
from we.tbs.models import WeTbsLicense, WeTbsRegulationPeriod
from we.product.models import WeCN8, WeUnuCode, WeHqppProduct, WeTbsProductLevel,\
    WeCN8PeriodProduct, WeTbsRegulationPeriodProduct, WeHqppRegulationPeriodProduct


class WeWeeeProductLevelCustomQuery(WEEEProductLevelCustomQuery):
    permissions = ('we_view_weee_category', )


class WeCN8WeeeProductLevelCustomQuery(AbstractWeCustomQuery):
    permissions = ('we_manage_cn8', )

    def tables(self):
        return (
            CustomQueryModel(WeCN8),
            CustomQueryModel(WEEEProductLevel, use_outerjoin=True, onclause=WeCN8.weee_product_level_id == WEEEProductLevel.id),
            CustomQueryModel(WeUnuCode, use_outerjoin=True, onclause=WeCN8.unu_code_id == WeUnuCode.id)
        )

    def fields(self):
        return (
            CustomQueryField(WeCN8.code, _('CN8'), key='cn8', is_filter=True, type=DATA_TYPE_STRING),
            CustomQueryField(WeCN8.description, _('CN8 description'), key='cn8_description', is_filter=True),
            CustomQueryField(WEEEProductLevel.label, _('WEEE product category'), key='weee_product_level', is_filter=True),
            CustomQueryField(WeUnuCode.code, _('UNU code'), key='unu_code', is_filter=True, type=DATA_TYPE_STRING),
            CustomQueryField(WeUnuCode.description, _('UNU description'), key='unu_description', is_filter=True),
            CustomQueryField(WEEEProductLevel.id, _('ID WEEE product category'), key='weee_product_level_id'),

        )


ParentLevel1 = aliased(WeTbsProductLevel)


class WeCN8PeriodProductCustomQuery(AbstractWeCustomQuery):
    permissions = ('we_manage_cn8_tbs_product_level', )

    def tables(self):
        return (
            CustomQueryModel(WeCN8PeriodProduct),
            CustomQueryModel(WeTbsRegulationPeriodProduct, use_outerjoin=True,
                             onclause=WeCN8PeriodProduct.period_product_id == WeTbsRegulationPeriodProduct.id),
            CustomQueryModel(WeTbsProductLevel, use_outerjoin=True,
                             onclause=WeTbsRegulationPeriodProduct.tbs_product_level_id == WeTbsProductLevel.id),
            CustomQueryModel(WeTbsRegulationPeriod, use_outerjoin=True,
                             onclause=WeTbsRegulationPeriodProduct.tbs_regulation_period_id == WeTbsRegulationPeriod.id),
            CustomQueryModel(WeTbsLicense, use_outerjoin=True,
                             onclause=WeTbsLicense.id == WeTbsProductLevel.tbs_id),
        )

    def fields(self):
        return (
            CustomQueryField(WeTbsLicense.name, _('TBS'), key='tbs_name', is_filter=True, type=DATA_TYPE_STRING),
            CustomQueryField(func.IF(func.count(WeCN8PeriodProduct.cn8_code)>1, func.concat(WeCN8PeriodProduct.cn8_code, ', ...'), WeCN8PeriodProduct.cn8_code), _('CN8'),key='cn8_code', use_when_separated=WeCN8PeriodProduct.cn8_code),
            CustomQueryField(WeTbsProductLevel.label, _('TBS local category'), key='tbs_product_level', is_filter=True),
            CustomQueryField( WeTbsRegulationPeriod.label, _('Regulation period'), key='regulation_period', is_filter=True, is_group_by=True),
            CustomQueryField(False,
                             _('B2B/B2C'), key='is_b2b', is_filter=True, type=DATA_TYPE_STRING,
                             use_when_separated=case([(WeCN8PeriodProduct.is_b2b == 1, 'B2C')],
                                                     else_=case([(WeCN8PeriodProduct.is_b2b == 2, 'B2B')], else_=None))),


        )

    # overridden to reset where
    def get_definitions(self, group_by, static_filters, applied_filters, no_empty_tables=True):
        if group_by is not None or applied_filters.has_key('regulation_period'):
            self.set_where(False)
        return AbstractWeCustomQuery.get_definitions(self, group_by, static_filters, applied_filters, no_empty_tables)

    def get_data(self, split_by, sort, asc, offset, limit, applied_filters, static_filters, row_count=True):
        if split_by is not None or applied_filters.has_key('regulation_period'):
            self.set_where(False)
        return AbstractWeCustomQuery.get_data(self, split_by, sort, asc, offset, limit, applied_filters, static_filters, row_count)

    def set_where(self, current_period_only=True):
        self.where = and_(WeTbsProductLevel.archived == 0, WeTbsRegulationPeriod.archived == 0)
        # by default the overview shows only current regulation period.
        if current_period_only:
            self.where = and_(
                self.where,
                or_(
                    func.isnull(WeTbsRegulationPeriod.id),
                    WeTbsRegulationPeriod.id.in_(self.get_regulation_periods())
                )
            )

    def get_regulation_periods(self, start_date=None):
        start_date = start_date if start_date is not None else self.today()
        sql = """
            SELECT r1.id
            FROM tblWeTbsRegulationPeriod as r1
            INNER JOIN (
              SELECT tbs_id, MAX(start_date) as sd
              FROM tblWeTbsRegulationPeriod
              WHERE start_date <= :start_date
                AND archived = 0
              GROUP BY tbs_id
            ) as r2
            ON r1.tbs_id = r2.tbs_id
              AND r1.start_date = r2.sd
            WHERE r1.tbs_id IN (%s)
        """
        tbss = [str(tbs.id) for tbs in self.get_tbs_licenses()]

        return [r.id for r in self.sa_session.execute(sql % ','.join(tbss), {'start_date': start_date})]

    def set_group_by(self):
        if self.separated:
            self.group_by = (WeTbsProductLevel.id, WeCN8PeriodProduct.cn8_code, )

        else:
            self.group_by = (WeTbsProductLevel.id, )


class WeTbsPriceSheetCustomQuery(AbstractWeCustomQuery):
    permissions = ('we_view_tbs_product_level',)  # Should this also be available for TBS users?

    def tables(self):
        return (
            CustomQueryModel(WeTbsProductLevel),
            CustomQueryModel(WeTbsLicense, use_outerjoin=True, onclause=WeTbsLicense.id == WeTbsProductLevel.tbs_id),
            CustomQueryModel(WeTbsRegulationPeriodProduct, use_outerjoin=True,
                             onclause=WeTbsProductLevel.id == WeTbsRegulationPeriodProduct.tbs_product_level_id),
            CustomQueryModel(WeTbsRegulationPeriod, use_outerjoin=True,
                             onclause=WeTbsRegulationPeriodProduct.tbs_regulation_period_id == WeTbsRegulationPeriod.id),
        )

    def fields(self):
        """
        We are using Regulation period position now
            CustomQueryField(
                func.ifnull(ParentLevel1.position, WeTbsProductLevel.position),
                "SORT", key='sort1'
            ),
            CustomQueryField(
                case([(ParentLevel1.position == None, 0)], else_=WeTbsProductLevel.position),
                "SORT2", key='sort2'
            ),
        """
        return (
            CustomQueryField(WeTbsRegulationPeriodProduct.position, _('Position'), key="rpp_position"),
            CustomQueryField(WeTbsProductLevel.id, _('Product level id'), key='id', sortable=False),
            CustomQueryField(WeTbsProductLevel.parent_level_id, _('Parent level id'), key='parent_level_id'),
            CustomQueryField(WeTbsLicense.license_id, _('TBS ID'), key='tbs_id'),
            CustomQueryField(WeTbsLicense.name, _('TBS'), key='tbs', sortable=False, is_filter=True),
            CustomQueryField(WeTbsProductLevel.label, _('Category label'), key='tbs_pl_label', is_filter=True),
            CustomQueryField(WeTbsProductLevel.description, _('Category description'), key='tbs_pl_description', is_filter=True),
            CustomQueryField(WeTbsProductLevel.invoice_basis, _('Basis for invoice'), key='invoice_basis', is_filter=True),
            CustomQueryField(WeTbsRegulationPeriodProduct.fee, _('Fee'), key="fee"),
            CustomQueryField(WeTbsRegulationPeriod.id, _('Regulation period id'), key="regulation_period_id"),
            CustomQueryField(WeTbsRegulationPeriod.label, _('Regulation period'), key="regulation_period", is_filter=True, is_group_by=True),
            CustomQueryField(WeTbsRegulationPeriod.start_date, _('Period start date'), key="regulation_period_start_date"),

        )

    # overridden to reset where
    def get_definitions(self, group_by, static_filters, applied_filters, no_empty_tables=True):
        if group_by is not None or applied_filters.has_key('regulation_period'):
            self.set_where(False)
        return AbstractWeCustomQuery.get_definitions(self, group_by, static_filters, applied_filters, no_empty_tables)

    def get_data(self, split_by, sort, asc, offset, limit, applied_filters, static_filters, row_count=True):
        if sort is None or sort == 'tbs_pl_label':
            self.order_by = ('tbs', 'regulation_period_id', 'rpp_position')
            sort = None
        if split_by is not None or applied_filters.has_key('regulation_period'):
            self.set_where(False)
        return AbstractWeCustomQuery.get_data(self, split_by, sort, asc, offset, limit, applied_filters, static_filters, row_count)

    def set_where(self, current_period_only=True):
        self.where = and_(WeTbsProductLevel.archived == 0, WeTbsRegulationPeriod.archived == 0)
        # by default the overview shows only current regulation period.

        if current_period_only:
            self.where = and_(
                self.where,
                or_(
                    func.isnull(WeTbsRegulationPeriod.id),
                    WeTbsRegulationPeriod.id.in_(self.get_regulation_periods())
                )
            )
            return
        # operators can see everyone, no need for slow IN-statements.
        if self.has_permissions('we_create_member_license'):
            return

        acc_license = self.get_default_group().license
        if isinstance(acc_license, WeTbsLicense):
            self.where = and_(
                self.where,
                WeTbsLicense.id == acc_license.id
            )
        if isinstance(acc_license, WeHqppLicense):
            self.where = and_(
                self.where,
                WeTbsLicense.id.in_([x.id for x in self.get_tbs_licenses()])
            )

    def get_regulation_periods(self, start_date=None):
        start_date = start_date if start_date is not None else self.today()
        sql = """
            SELECT r1.id
            FROM tblWeTbsRegulationPeriod as r1
            INNER JOIN (
              SELECT tbs_id, MAX(start_date) as sd
              FROM tblWeTbsRegulationPeriod
              WHERE start_date <= :start_date
                AND archived = 0
              GROUP BY tbs_id
            ) as r2
            ON r1.tbs_id = r2.tbs_id
              AND r1.start_date = r2.sd
            WHERE r1.tbs_id IN (%s)
        """
        tbss = [str(tbs.id) for tbs in self.get_tbs_licenses()]

        return [r.id for r in self.sa_session.execute(sql % ','.join(tbss), {'start_date': start_date})]

    group_by = (WeTbsProductLevel.id, )


Tbs = aliased(License)
TbsCompany = aliased(Company)
class WeHqppProductCustomQuery(AbstractWeCustomQuery):
    permissions = ('we_view_products', )

    #TODO: show only the rows which have a b2b value for tbs with b2b setting enabled else show rows with B2B None
    def tables(self):
        # get user license
        # if tbs set outerjoin = False
        is_outerjoin = True
        acc_license = self.get_default_group().license
        if isinstance(acc_license, WeTbsLicense):
            is_outerjoin = False
        # regulation
        return (
            CustomQueryModel(WeHqppProduct),
            CustomQueryModel(WeHqppLicense, onclause=WeHqppProduct.hqpp_id == WeHqppLicense.license_id),
            CustomQueryModel(WeHqppRegulationPeriodProduct,
                             onclause=WeHqppProduct.id == WeHqppRegulationPeriodProduct.product_id, use_outerjoin=is_outerjoin),
            CustomQueryModel(WeTbsRegulationPeriodProduct,
                             onclause=WeHqppRegulationPeriodProduct.tbs_regulation_period_product_id ==
                                      WeTbsRegulationPeriodProduct.id, use_outerjoin=is_outerjoin),
            CustomQueryModel(WeTbsProductLevel,
                             onclause=WeTbsRegulationPeriodProduct.tbs_product_level_id == WeTbsProductLevel.id, use_outerjoin=is_outerjoin),
            CustomQueryModel(WeTbsRegulationPeriod,
                             onclause=WeTbsRegulationPeriodProduct.tbs_regulation_period_id == WeTbsRegulationPeriod.id,
                             use_outerjoin=is_outerjoin),
            CustomQueryModel(Tbs, onclause=WeTbsProductLevel.tbs_id == Tbs.id, use_outerjoin=is_outerjoin),
            CustomQueryModel(WeLpp, onclause=and_(WeLpp.hqpp_id == WeHqppLicense.id,
                                                  WeLpp.tbs_id == Tbs.id), use_outerjoin=is_outerjoin),
            CustomQueryModel(WeCN8, onclause=WeHqppProduct.cn8 == WeCN8.code, use_outerjoin=True),
            CustomQueryModel(WeUnuCode, onclause=WeCN8.unu_code_id == WeUnuCode.id, use_outerjoin=True),

            CustomQueryModel(Company, onclause=WeHqppLicense.company_id == Company.id, use_outerjoin=True),
            CustomQueryModel(TbsCompany, onclause=Tbs.company_id == TbsCompany.id, use_outerjoin=True),
            CustomQueryModel(CompanyAddress, onclause=TbsCompany.id == CompanyAddress.company_id, use_outerjoin=True),
            CustomQueryModel(Address, onclause=Address.id == CompanyAddress.address_id, use_outerjoin=True),
            CustomQueryModel(Country, onclause=Address.country_id == Country.id, use_outerjoin=True),

        )

    def fields(self):
        return (
            CustomQueryField(Company.name, _('Producer'), key='hqpp_name', is_filter=True),
            CustomQueryField(WeHqppProduct.id, _('Product id'), key='product_id'),
            CustomQueryField(WeHqppProduct.name, _('Product name'), key='product_name', is_filter=True),
            CustomQueryField(WeHqppProduct.hqpp_id, _('HQPP id'), key='hqpp_id'),
            CustomQueryField(WeHqppProduct.sku, _('SKU code'), key='sku', is_filter=True),
            CustomQueryField(WeCN8.code, _('CN8 code'), key='cn8', is_filter=True),
            CustomQueryField(WeUnuCode.code, _('UNU code'), key='unu_code', is_filter=True),

            # these fields should only be shown on separate usage
            CustomQueryField(False, _('Local category id'), key='tbs_product_level_id', use_when_separated=WeTbsRegulationPeriodProduct.tbs_product_level_id),
            CustomQueryField(False, _('Local producer partner'), key='lpp_name', use_when_separated=WeLpp.name),
            CustomQueryField(False, _('TBS'), key='tbs_name', use_when_separated=Tbs.name),
            CustomQueryField(False, _('Country'), key='tbs_country', use_when_separated=Country.name),
            CustomQueryField(False, _('TBS local category'), key='tbs_product_level', use_when_separated=WeTbsProductLevel.label),
            CustomQueryField(False, _('TBS regulation period'), is_filter=True, key='tbs_regulation_period', use_when_separated=WeTbsRegulationPeriod.label, is_group_by=True),
            CustomQueryField(False, _('B2B/B2C'), key='b2b-b2c', use_when_separated=case(
                [(WeHqppRegulationPeriodProduct.is_b2b == 1, 'B2C')],
                else_=case([(WeHqppRegulationPeriodProduct.is_b2b == 2, 'B2B')], else_=None))),

            CustomQueryField(WeHqppProduct.weight_per_unit, _('Weight per unit (kg)'), key='weight_per_unit', is_filter=True, type=DATA_TYPE_DECIMAL),
            CustomQueryField(WeHqppProduct.length_per_unit, _('Length per unit (cm)'), key='length_per_unit', is_filter=True, type=DATA_TYPE_DECIMAL),
            CustomQueryField(WeHqppProduct.height_per_unit, _('Height per unit (cm)'), key='height_per_unit', is_filter=True, type=DATA_TYPE_DECIMAL),
            CustomQueryField(WeHqppProduct.width_per_unit, _('Width per unit (cm)'), key='width_per_unit', is_filter=True, type=DATA_TYPE_DECIMAL),
            CustomQueryField(WeHqppProduct.diagonal, _('Diagonal (inches)'), key='diagonal', is_filter=True, type=DATA_TYPE_DECIMAL),
            CustomQueryField(WeHqppProduct.sales_price, _('Sales price'), key='sales_price', is_filter=True, type=DATA_TYPE_DECIMAL),

            CustomQueryField(WeHqppProduct.weight_cat1, _('Weight category 1'), key='weight_cat1', is_filter=True, type=DATA_TYPE_DECIMAL),
            CustomQueryField(WeHqppProduct.weight_cat2, _('Weight category 2'), key='weight_cat2', is_filter=True, type=DATA_TYPE_DECIMAL),
            CustomQueryField(WeHqppProduct.weight_cat3, _('Weight category 3'), key='weight_cat3', is_filter=True, type=DATA_TYPE_DECIMAL),
            CustomQueryField(WeHqppProduct.weight_cat4, _('Weight category 4'), key='weight_cat4', is_filter=True, type=DATA_TYPE_DECIMAL),
            CustomQueryField(WeHqppProduct.weight_documentation, _('Weight documentation'), key='weight_documentation',
                             is_filter=True, type=DATA_TYPE_DECIMAL),
            CustomQueryField(WeHqppProduct.weight_package, _('Weight package'), key='weight_package', is_filter=True, type=DATA_TYPE_DECIMAL),
            CustomQueryField(WeHqppProduct.weight_batteries, _('Weight batteries'), key='weight_batteries', is_filter=True, type=DATA_TYPE_DECIMAL),

            CustomQueryField(WeHqppProduct.description, _('Description'), key='description')
        )

    def set_where(self):
        # operators can see everyone, no need for slow IN-statements.
        if self.has_permissions('we_create_member_license'):
            return
        acc_license = self.get_default_group().license
        if isinstance(acc_license, WeTbsLicense):
            self.where = Tbs.id == acc_license.id
        if isinstance(acc_license, WeHqppLicense):
            self.where = WeHqppLicense.id.in_([x.id for x in self.get_member_licenses()])

    def get_data(self, split_by, sort, asc, offset, limit, applied_filters, static_filters, row_count=True):
        if sort is None:
            self.order_by = (WeHqppLicense.id, WeHqppProduct.id, )
            sort = None
        return AbstractWeCustomQuery.get_data(self, split_by, sort, asc, offset, limit, applied_filters, static_filters, row_count)

    def set_group_by(self):
        if self.separated:
            self.group_by = None
        else:
            self.group_by = (WeHqppProduct.id,)

class FileImportCustomQuery(AbstractFileImportCustomQuery):
    def set_where(self):

        allowed = []
        if self.has_permissions('we_manage_tbs_product_level'):
            allowed.append(TbsPriceSheetLineImporter.IMPORT_FILE_TYPE)
        if self.has_permissions('we_create_products'):
            allowed.append(HqppProductLineImporter.IMPORT_FILE_TYPE)
        if self.has_permissions('we_manage_cn8'):
            allowed.append(CN8LineImporter.IMPORT_FILE_TYPE)
        if self.has_permissions('we_manage_cn8_tbs_product_level'):
            allowed.append(CN8ToTbsProductLevelLineImporter.IMPORT_FILE_TYPE)
        if self.has_permissions('we_manage_declaration'):
            allowed.append(WeDeclarationImporter.IMPORT_FILE_TYPE)
        if len(allowed) == 0:
            raise PublicSecurityException(_('You do not have access to any import files'))

        self.where = ImportFileType.name.in_(allowed)


custom_queries = (
    WeWeeeProductLevelCustomQuery,
    WeCN8WeeeProductLevelCustomQuery,
    WeCN8PeriodProductCustomQuery,
    WeTbsPriceSheetCustomQuery,
    WeHqppProductCustomQuery,
    FileImportCustomQuery,

)
