# Copyright (c) 2016 BaseSoft B.V.
# See LICENSE for details.
from sqlalchemy.sql.expression import and_

from basesoft.scheduler.crontab import CronTab
from basesoft.scheduler.task import ScheduleTask
from basesoft.web.contrib.file.schedule import AbstractImportFileTask

from we.product.controllers import WeProductController
from we.product.importers import TbsPriceSheetAbstractImporter, HqppProductAbstractImporter, \
    CN8AbstractImporter, CN8ToTbsProductLevelAbstractImporter
from we.tbs.models import WeTbsRegulationPeriod


class HqppProductSheetTask(AbstractImportFileTask):
    """
    Background action for importing producer product sheet.
    """
    INTERVAL = 0
    ALLOW_DUPLICATES = True  # Need to enable this in order to queue
    QUEUE_IF_EXISTS = True
    controller = WeProductController
    importer = HqppProductAbstractImporter


class TbsPriceSheetImportTask(AbstractImportFileTask):
    """
    Background action for importing TBS local categories and fees from a saved file.
    """
    INTERVAL = 0
    ALLOW_DUPLICATES = True  # Need to enable this in order to queue
    QUEUE_IF_EXISTS = True
    controller = WeProductController
    importer = TbsPriceSheetAbstractImporter


class CN8ToTbsProductLevelTask(AbstractImportFileTask):
    """
    Background action for importing suggestions on CN8 linked to TBS local categories from a saved file.
    """
    INTERVAL = 0
    ALLOW_DUPLICATES = True  # Need to enable this in order to queue
    QUEUE_IF_EXISTS = True
    controller = WeProductController
    importer = CN8ToTbsProductLevelAbstractImporter


class CN8ToWeeeCategoryTask(AbstractImportFileTask):
    """
    Background action for importing saved files with CN8, UNU, WEEE category.
    """
    INTERVAL = 0
    ALLOW_DUPLICATES = True  # Need to enable this in order to queue
    QUEUE_IF_EXISTS = True
    controller = WeProductController
    importer = CN8AbstractImporter


class RegulationPeriodChangeTask(ScheduleTask):
    INTERVAL = CronTab('05 0 * * *')  # Runs ever day at 5 minutes past midnight.

    def execute(self):

        c = WeProductController(self.sa_session)

        periods = self.sa_session.query(WeTbsRegulationPeriod).filter(
            and_(
                WeTbsRegulationPeriod.is_processed == False,
                WeTbsRegulationPeriod.archived == 0
            )
        ).all()

        for period in periods:
            if period.start_date != period.tbs.domain.today():
                continue

            self.logger.info('New regulation period (%s) for TBS %s' % (period.title, period.tbs))
            c.compare_regulation_period_with_previous(period, False)

tasks = (HqppProductSheetTask, TbsPriceSheetImportTask, CN8ToWeeeCategoryTask, RegulationPeriodChangeTask, CN8ToTbsProductLevelTask, )