'''
Created on Dec 17, 2014

@author: marten
'''
from basesoft.web.contrib.license.controllers import LicenseController
from basesoft.web.core.util.controllers import transaction_decorator

from we.core.controllers import AbstractWeController
from we.member.models import WeContact
from we.tbs.models import WeTbsLicense
from basesoft.web.contrib.license.models import Domain
from basesoft.web.contrib.contact.models import Person, Company

class WeTbsController(LicenseController, AbstractWeController):

    _instance_class = WeContact

    def get_all_tbs(self):
        return self.sa_session.query(WeTbsLicense).order_by(WeTbsLicense.name).all()

    def get_tbses_per_hqpp(self, contains, value, limit):

        licenses = self.sa_session.query(WeTbsLicense)

        if value not in [None, '']:
            return [licenses.filter(WeTbsLicense.license_id == int(value)).one()]

        if contains not in [None, '']:
            licenses = licenses.filter(WeTbsLicense.name.like('%%%s%%' % contains))
        return licenses.order_by(WeTbsLicense.name).limit(limit).all()

    def get_all_own_tbs(self):
        """
        Get all TBS to which the current user has access to.
        :return:
        """
        return self.get_tbs_licenses()

    def load_tbs(self, _id):
        assert isinstance(_id, (int, long))
        self.assert_permissions('we_tbs_license')

        return self.load_license_state(_id)

    def save_or_update_tbs(self, _id, state):
        if _id is None:
            _license = self.save_tbs(state)
            self.grant_from_role('WE TBS', _license, _license.groups[0], link_role=True)
            return _license
        state['dbID'] = _id
        return self.save_tbs(state)

    @transaction_decorator
    def save_tbs(self, state):
        # A TBS is an extension of basesoft License.

        assert isinstance(state, dict)
        self.assert_permissions('we_tbs_license')

        state['license'] = {
            'polymorphic_type': 'WeTbsLicense',
            'name': state['company']['name'],
            'expiration_date': None,
            'is_default': 0
        }

        state['domain'] = {
            'name': self.get_default_group().license.domain.name
        }

        state['group'] = {
            'name': 'default'
        }
        state['permissions'] = []

        _license = self.save_license_from_state(state)
        self.grant_from_role('WE TBS', _license, _license.groups[0], link_role=True)
        return _license

    @transaction_decorator
    def save_or_update_license(self, _id, name, domain, polymorphic_type, contact, company, expiration_date,
                               is_default=False, permissions=[], usage_options=[], state=None):

        assert isinstance(domain, Domain)
        assert isinstance(contact, Person)
        assert isinstance(company, Company)

        self.assert_permissions("we_tbs_license")

        if _id is not None:

            # a license type cannot be changed; id and polymorphic type must match otherwise
            # next query will not have any results
            _license = self.sa_session.query(polymorphic_type).filter(polymorphic_type.id == _id)
            _license = _license.one()
        else:
            _license = polymorphic_type()
            _license.domain = domain

        assert isinstance(_license, WeTbsLicense)

        _license.name = unicode(name)
        _license.contact_person = contact
        _license.expiration_date = expiration_date
        _license.is_default = is_default
        _license.company = company

        # And save the permissions to the license
        if len(permissions) > 0:
            self.save_permissions_to_license(_license, permissions, None)

        self.sa_session.add(_license)
        return _license
